CREATE OR REPLACE PROCEDURE CTB_DFE_CONTAB(
   P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_NRLANCTONF   IN NOTAFISCAL.NRLANCTONF%TYPE,
   P_NRLANCCTB    OUT LANCCONT.NRLANCCONT%TYPE)
AS
   V_NRLANCTEMP VARCHAR2(20);
   V_DSHISTLANC LANCCONT.DSHISTLANC%TYPE;
   V_COUNT INT;
   V_ACUMULA VARCHAR(01);
   V_FATOR INT;
   V_VRLANCTO NUMBER;
   V_VRTEMP NUMBER;
   V_NUMPARTIDA INT;
   V_IDCOME VARCHAR2(01);
   V_IDCCUS VARCHAR2(01);
   V_CDHISTPADR HISTPADR.CDHISTPADR%TYPE;
   V_VRSEQLANC LANCCONT.NRSEQLANC%TYPE;
   V_CDCENTCUST CENTCUST.CDCENTCUST%TYPE;
   V_CONTCCUS VARCHAR2(01);
   V_CDCOMERCIAL FORNECEDOR.CDFORNECED%TYPE;
   V_NRSEQLANC LANCCONT.NRSEQLANC%TYPE;
   V_VRINDUSTRIA LANCCONT.VRLANCAMENT%TYPE;

   PROCEDURE NUMEROPARTIDA(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                           P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                           P_CDFORNECED IN FORNECEDOR.CDFORNECED%TYPE,
                           P_CDCLIENTE IN CLIENTE.CDCLIENTE%TYPE)
   AS
   BEGIN
      IF V_NUMPARTIDA IS NULL THEN
         /* NUMERO DA PARTIDA */
         BEGIN
            SELECT TO_NUMBER(MAX(NRSEQCTBTEMP))
              INTO V_NUMPARTIDA
              FROM LANCCTBTEMP
             WHERE NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDEMPRESA = P_CDEMPRESA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NUMPARTIDA := 0;
         END;

         /* CONTROLA SE GERA OS RATEIOS - CONTROLA SE AS PARTIDAS ACUMULAM */
         SELECT IDCONTGERE, IDACUMVRCONT
           INTO V_CONTCCUS, V_ACUMULA
           FROM CONTOPECTBL
          WHERE CDEMPRESA = P_CDEMPRESA;

         /* BUSCA O CODIGO COMERCIAL */
         BEGIN
            SELECT DECODE(P_IDORIGDESTNF, 'F', P_CDFORNECED, DECODE(P_IDORIGDESTNF, 'C', P_CDCLIENTE, NULL))
              INTO V_CDCOMERCIAL
              FROM DUAL;
          EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDCOMERCIAL := NULL;
         END;
      END IF;
   END;

   PROCEDURE DESPESA_DIFALIQUOTA(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                                 P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                                 P_CDFILIRECE IN FILIAL.CDFILIAL%TYPE,
                                 P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                                 P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                                 P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                                 P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                                 P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                                 P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                                 P_DTENTRSAID IN NOTAFISCAL.DTENTRSAID%TYPE)
   AS
   BEGIN
      NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);
      V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIRECE, P_CDEMPRESA);
      FOR C_DESPDIFALIQ IN
         (SELECT (ROUND(IMP.VRIMPOSTOIT, 2) * -1) VRIMPOSTOIT,
                 CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCCTBLDESP
            FROM NOTAFISCAL NF, ITEMPROIMPOS IMP, ITEMPRODUTO IT, IMPOSTO I
           WHERE IMP.CDFILIAL    = P_CDFILIAL      AND
                 IMP.NRLANCTONF  = P_NRLANCTONF    AND
                 IMP.CDFILIAL    = NF.CDFILIAL    AND
                 IMP.NRLANCTONF  = NF.NRLANCTONF  AND
                 IMP.NRSEQITPROD = IT.NRSEQITPROD AND
                 IMP.NRLANCTONF  = IT.NRLANCTONF  AND
                 IMP.CDFILIAL    = IT.CDFILIAL    AND
                 IMP.CDIMPOSTO   = I.CDIMPOSTO    AND
                 IMP.CDFISCALIT  IN ('2', '3')    AND
                 IMP.NRITPROIMPOS = '3'           AND
                 I.IDTIPCONTAB   = 'D'            AND
                 IDIMPORECUP     = 'S'            AND
                 IDDIFALIQCUSPRI = 'N') LOOP
         BUSCA_TPCOMERCIAL(C_DESPDIFALIQ.CDCCTBLDESP, V_IDCOME, V_IDCCUS);
         BEGIN
            SELECT COUNT(*), NRSEQCTBTEMP
              INTO V_COUNT, V_NRSEQLANC
              FROM LANCCTBTEMP
             WHERE NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDEMPRESA = P_CDEMPRESA
               AND CDCONTCTBL = C_DESPDIFALIQ.CDCCTBLDESP
              GROUP BY NRSEQCTBTEMP;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
               V_NRSEQLANC := NULL;
         END;

         IF V_COUNT = 0 THEN
            V_NUMPARTIDA := V_NUMPARTIDA + 1;
            IF NVL(V_IDCOME, 'N') = 'N' THEN
               V_CDCOMERCIAL := NULL;
            END IF;
            INSERT INTO LANCCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
            VALUES
               (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                C_DESPDIFALIQ.CDCCTBLDESP, V_CDHISTPADR, V_DSHISTLANC, P_DTENTRSAID, C_DESPDIFALIQ.VRIMPOSTOIT, V_IDCOME, V_CDCOMERCIAL, 'DFE');
            IF V_CONTCCUS <> 'N' THEN
               IF V_IDCCUS <> 'O' THEN
                  INSERT INTO RATECTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                      CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                  VALUES
                     (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIRECE,
                      V_CDCENTCUST, C_DESPDIFALIQ.VRIMPOSTOIT, P_DTENTRSAID, '00001');
               END IF;
            END IF;
         ELSE
            UPDATE LANCCTBTEMP
               SET VRLANCCTBTEMP = VRLANCCTBTEMP + C_DESPDIFALIQ.VRIMPOSTOIT
             WHERE CDEMPRESA = P_CDEMPRESA
               AND NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDCONTCTBL = C_DESPDIFALIQ.CDCCTBLDESP;

            BEGIN
               SELECT COUNT(*), L.NRSEQCTBTEMP
                 INTO V_COUNT, V_NRSEQLANC
                 FROM LANCCTBTEMP L, RATECTBTEMP R
                WHERE L.NRLANCCTBTEMP = V_NRLANCTEMP
                  AND L.CDEMPRESA = P_CDEMPRESA
                  AND L.CDCONTCTBL = C_DESPDIFALIQ.CDCCTBLDESP
                  AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                  AND L.NRSEQCTBTEMP = R.NRSEQCTBTEMP
                  AND L.CDEMPRESA = R.CDEMPRESA
                  AND R.CDFILIAL = P_CDFILIRECE
                  AND R.CDCENTCUST = V_CDCENTCUST;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                   V_COUNT := 0;
                   V_NRSEQLANC := NULL;
            END;

            IF V_COUNT = 0 THEN
               INSERT INTO RATECTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                   CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
               VALUES
                  (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIRECE,
                   V_CDCENTCUST, C_DESPDIFALIQ.VRIMPOSTOIT, P_DTENTRSAID, '00001');
            ELSE
               UPDATE RATECTBTEMP
                  SET VRRATECTBTEMP = VRRATECTBTEMP + C_DESPDIFALIQ.VRIMPOSTOIT
                WHERE CDEMPRESA = P_CDEMPRESA
                  AND NRLANCCTBTEMP = V_NRLANCTEMP
                  AND NRSEQCTBTEMP = V_NRSEQLANC
                  AND CDFILIAL = P_CDFILIRECE
                  AND CDCENTCUST = V_CDCENTCUST;
            END IF;
         END IF;
      END LOOP;
   END;


   PROCEDURE CONTABILIZA_DESCONTO(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                                  P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                                  P_CDFILIRECE IN FILIAL.CDFILIAL%TYPE,
                                  P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                                  P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                                  P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                                  P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                                  P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                                  P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                                  P_DTENTRSAID IN NOTAFISCAL.DTENTRSAID%TYPE)
   AS
   BEGIN
      NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);
      V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIRECE, P_CDEMPRESA);

      /*-------------------------------------------------------------*/
      /* FAZ O SOMATORIO DOS DESCONTOS DA NOTA FISCAL                */
      /*-------------------------------------------------------------*/
      FOR C_DESC IN
         (SELECT ROUND(SUM(VRDESCITEM),2) AS VRITEMENCA, CDCTBLENTREN
            FROM ITEMPRODUTO I,
                 (SELECT CDCONTDITEN AS CDCTBLENTREN
                    FROM PARAMCTBL
                   WHERE CDCONTDITEN IS NOT NULL) E
            WHERE I.CDFILIAL   = P_CDFILIAL   AND
                  I.NRLANCTONF = P_NRLANCTONF
            GROUP BY CDCTBLENTREN
            HAVING ROUND(SUM(VRDESCITEM),2) > 0)LOOP
         BUSCA_TPCOMERCIAL(C_DESC.CDCTBLENTREN, V_IDCOME, V_IDCCUS);

         BEGIN
            SELECT COUNT(*), NRSEQCTBTEMP
              INTO V_COUNT, V_NRSEQLANC
              FROM LANCCTBTEMP
             WHERE CDEMPRESA = P_CDEMPRESA
               AND NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDCONTCTBL = C_DESC.CDCTBLENTREN
             GROUP BY NRSEQCTBTEMP;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            V_NUMPARTIDA := V_NUMPARTIDA + 1;
            IF NVL(V_IDCOME, 'N') = 'N' THEN
               V_CDCOMERCIAL := NULL;
            END IF;
            INSERT INTO LANCCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
            VALUES
               (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                C_DESC.CDCTBLENTREN, V_CDHISTPADR, V_DSHISTLANC, P_DTENTRSAID, C_DESC.VRITEMENCA, V_IDCOME, V_CDCOMERCIAL, 'DFE');

            IF V_CONTCCUS <> 'N' THEN
               IF V_IDCCUS <> 'O' THEN
                  INSERT INTO RATECTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                      CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                  VALUES
                     (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIRECE,
                      V_CDCENTCUST, C_DESC.VRITEMENCA, P_DTENTRSAID, '00001');
               END IF;
            END IF;
         ELSE
            UPDATE LANCCTBTEMP
               SET VRLANCCTBTEMP = VRLANCCTBTEMP + C_DESC.VRITEMENCA
             WHERE CDEMPRESA = P_CDEMPRESA
               AND NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDCONTCTBL = C_DESC.CDCTBLENTREN;
            IF V_CONTCCUS <> 'N' THEN
               IF V_IDCCUS <> 'O' THEN
                  BEGIN
                     SELECT COUNT(*), L.NRSEQCTBTEMP
                       INTO V_COUNT, V_NRSEQLANC
                       FROM LANCCTBTEMP L, RATECTBTEMP R
                      WHERE L.CDEMPRESA = P_CDEMPRESA
                        AND L.NRLANCCTBTEMP = V_NRLANCTEMP
                        AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                        AND L.CDEMPRESA = R.CDEMPRESA
                        AND L.CDCONTCTBL = C_DESC.CDCTBLENTREN
                        AND L.NRLANCCTBTEMP = L.NRLANCCTBTEMP
                        AND R.CDFILIAL = P_CDFILIRECE
                        AND R.CDCENTCUST = V_CDCENTCUST
                      GROUP BY L.NRSEQCTBTEMP;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_COUNT := 0;
                        V_NRSEQLANC := NULL;
                  END;

                  IF V_COUNT = 0 THEN
                     INSERT INTO RATECTBTEMP
                        (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                         CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                     VALUES
                        (P_CDEMPRESA, V_NRLANCTEMP, V_NRSEQLANC, P_CDFILIRECE,
                         V_CDCENTCUST, C_DESC.VRITEMENCA, P_DTENTRSAID, '00001');
                  ELSE
                     UPDATE RATECTBTEMP
                        SET VRRATECTBTEMP = VRRATECTBTEMP
                      WHERE CDEMPRESA = P_CDEMPRESA
                        AND NRLANCCTBTEMP = V_NRLANCTEMP
                        AND NRSEQCTBTEMP = V_NRSEQLANC
                        AND CDFILIAL = P_CDFILIRECE
                        AND CDCENTCUST = V_CDCENTCUST;
                  END IF;
               END IF;
            END IF;
         END IF;
      END LOOP;
   END;

   PROCEDURE CONTABILIZA_ENCARGO(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                                 P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                                 P_CDFILIRECE IN FILIAL.CDFILIAL%TYPE,
                                 P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                                 P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                                 P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                                 P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                                 P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                                 P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                                 P_DTENTRSAID IN NOTAFISCAL.DTENTRSAID%TYPE)

   AS
   BEGIN
      FOR C_ENC IN
        (SELECT ROUND(VRITEMENCA, 2) VRITEMENCA, E.IDENCARGO, CDCTBLENTREN
          FROM ITEMENCARGO IE, ENCARGO E
         WHERE IE.CDFILIAL   = P_CDFILIAL
           AND IE.NRLANCTONF = P_NRLANCTONF
           AND IE.CDENCARGO  = E.CDENCARGO
           AND IE.VRITEMENCA > 0
           AND E.IDENCARGO IN ('A','D')
           AND (DECODE(P_IDENTRSAID, 'E', CDCTBLENTREN, CDCTBLSAIDEN) IS NOT NULL))LOOP
         BUSCA_TPCOMERCIAL(C_ENC.CDCTBLENTREN, V_IDCOME, V_IDCCUS);
         V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIRECE, P_CDEMPRESA);
         IF C_ENC.IDENCARGO = 'A' THEN
            V_VRTEMP := - C_ENC.VRITEMENCA * V_FATOR;
         ELSE
            V_VRTEMP := (C_ENC.VRITEMENCA * V_FATOR);
         END IF;

         NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);

         IF C_ENC.CDCTBLENTREN IS NOT NULL THEN
            BEGIN
               SELECT COUNT(*)
                 INTO V_COUNT
                 FROM LANCCTBTEMP
                WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDCONTCTBL = C_ENC.CDCTBLENTREN;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_COUNT := 0;
            END;
            IF V_COUNT = 0 THEN
               V_NUMPARTIDA := V_NUMPARTIDA + 1;
               IF NVL(V_IDCOME, 'N') = 'N' THEN
                  V_CDCOMERCIAL := NULL;
               END IF;
               INSERT INTO LANCCTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                   CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                   VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
               VALUES
                  (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                   C_ENC.CDCTBLENTREN, V_CDHISTPADR, V_DSHISTLANC, P_DTENTRSAID, V_VRTEMP, V_IDCOME, V_CDCOMERCIAL, 'DFE');

               IF V_CONTCCUS <> 'N' THEN
                  IF V_IDCCUS <> 'O' THEN
                     INSERT INTO RATECTBTEMP
                        (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                         CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                     VALUES
                        (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIRECE,
                         V_CDCENTCUST, V_VRTEMP, P_DTENTRSAID, '00001');
                  END IF;
               END IF;
            ELSE
               UPDATE LANCCTBTEMP
                  SET VRLANCCTBTEMP = VRLANCCTBTEMP + V_VRTEMP
                WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDCONTCTBL = C_ENC.CDCTBLENTREN;
               IF V_CONTCCUS <> 'N' THEN
                  IF V_IDCCUS <> 'O' THEN
                     BEGIN
                        SELECT COUNT(*), L.NRSEQCTBTEMP
                          INTO V_COUNT, V_NRSEQLANC
                          FROM LANCCTBTEMP L, RATECTBTEMP R
                         WHERE L.NRLANCCTBTEMP = V_NRLANCTEMP
                           AND L.CDEMPRESA = P_CDEMPRESA
                           AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                           AND L.CDEMPRESA = R.CDEMPRESA
                           AND L.CDCONTCTBL = C_ENC.CDCTBLENTREN
                           AND R.CDFILIAL = P_CDFILIRECE
                           AND R.CDCENTCUST = V_CDCENTCUST
                         GROUP BY L.NRSEQCTBTEMP;
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_COUNT := 0;
                           V_NRSEQLANC := NULL;
                     END;

                     IF V_COUNT = 0 THEN
                        INSERT INTO RATECTBTEMP
                           (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                            CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                        VALUES
                           (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIRECE,
                            V_CDCENTCUST, V_VRTEMP, P_DTENTRSAID, '00001');
                     ELSE
                        UPDATE RATECTBTEMP
                           SET VRRATECTBTEMP = VRRATECTBTEMP + V_VRTEMP
                         WHERE CDEMPRESA = P_CDEMPRESA
                           AND NRLANCCTBTEMP = V_NRLANCTEMP
                           AND NRSEQCTBTEMP = V_NRSEQLANC
                           AND CDFILIAL = P_CDFILIRECE
                           AND CDCENTCUST = V_CDCENTCUST;
                     END IF;
                  END IF;
               END IF;
            END IF;
         ELSE
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (P_CDEMPRESA, V_NRLANCTEMP, '', '015');
         END IF;
      END LOOP;
   END;
BEGIN
   V_VRLANCTO := 0;
   V_VRTEMP := 0;
   FOR C_NOTAFISCAL IN
      (SELECT NF.CDFILIAL,      NF.NRLANCTONF,    NF.CDEMPRESA,
              NF.IDDOCFINANC,   NF.NRNOTAFISC,    NF.CDCLIENTE,
              NF.CDFORNECED,    NF.NRLANCTBDOC,   NF.VRNOTAFISC,
              NF.CDFILIEMIT,    NF.CDTIPOOPER,    TP.IDCONTABOPER,
              TP.IDOPERDEVONF,  NF.IDORIGDESTNF,  NF.VRPRODUTOS,
              NF.CDFILIRECE,    NF.DTENTRSAID,    NF.CDINSCESTA,
              NF.DTEMISSAO,     C.IDPRIPUBCLIE,   NF.IDENTRSAIDOP,
              NVL(FI.CDCENTCUST, EP.CDCENTCUST) AS CDCENTCUST,
              TP.CDTPCTPAGAR AS CDTPCTPAGOP, TP.CDTPCTRECEB AS CDTPCTRECOP,
              TC.CDCLASSFINA AS CDCLAFINOP, FO.DTINIVRPDESC, NF.CDSERIEENTR,
              NF.CDSERIE, NF.IDTPIJURNF, NF.NRINSJURNF, TP.IDDTCONTAB,
              TP.IDCOAVRIMPVRPROD
         FROM NOTAFISCAL NF, FILIAL FI, EMPRESA EP,
              TIPOOPERAC TP, TIPOCTPAGAR TC, FORNECEDOR FO, CLIENTE C
        WHERE NF.CDFILIAL     = P_CDFILIAL        AND
              NF.NRLANCTONF   = P_NRLANCTONF      AND
              NF.CDEMPRESA    = EP.CDEMPRESA      AND
              NF.IDENTRSAIDOP = TP.IDENTRSAIDOP   AND
              NF.CDTIPOOPER   = TP.CDTIPOOPER     AND
              TP.CDTPCTPAGAR  = TC.CDTPCTPAGAR(+) AND
              DECODE(NF.IDENTRSAIDOP,'E', NF.CDFILIRECE, NF.CDFILIEMIT) = FI.CDFILIAL AND
              NF.CDFORNECED   = FO.CDFORNECED(+)  AND
              NF.CDCLIENTE    = C.CDCLIENTE(+))LOOP

      IF C_NOTAFISCAL.IDCONTABOPER <> 'N' THEN
         V_NRLANCTEMP := C_NOTAFISCAL.NRLANCTBDOC;
         IF V_NRLANCTEMP IS NULL THEN
            /* PEGA UM NOVO NMERO DE LANAMENTO CONTBIL, CASO O LANAMENTO SEJA NOVO */
            NOVO_CODIGO_NT('LANCCONTABIL' || C_NOTAFISCAL .CDEMPRESA || TO_CHAR(C_NOTAFISCAL.DTENTRSAID,'YYYYMM'), V_NRLANCTEMP);
            V_NRLANCTEMP := TO_CHAR(C_NOTAFISCAL.DTENTRSAID,'YYYYMM')||SUBSTR(V_NRLANCTEMP, 15, 6);
         END IF;

         /* IDCOAVRIMPVRPROD -> CONTABILIZA SOMENTE VALOR DOS IMPOSTOS NA PARTIDA DO PRODUTO */
         IF ((C_NOTAFISCAL.CDTIPOOPER = '14') OR (C_NOTAFISCAL.IDCOAVRIMPVRPROD = 'S')) THEN
            V_FATOR := -1;
         ELSE
            V_FATOR := 1;
         END IF;

         /* DELETA O ANTIGO LANAMENTO CONTBIL PARA REFAZE-LO */
         IF V_NRLANCTEMP IS NOT NULL THEN
            DELETE
              FROM LANCCONT
             WHERE NRLANCCONT = V_NRLANCTEMP
               AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;

            DELETE
              FROM LANCCCUSTO
             WHERE NRLANCCONT = V_NRLANCTEMP
               AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;
         END IF;

         IF C_NOTAFISCAL.IDCOAVRIMPVRPROD = 'N' THEN
            /* CONTABILIZA O VALOR DA PARTIDA DE FORNECEDOR */
            CTB_DFE_FORNECEDOR(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_FATOR, V_VRLANCTO, V_VRINDUSTRIA);

            /* CONTABILIZA OS DIFERENCIAIS DE ALIQUOTA */
            CTB_DFE_DIFALIQUOTA(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_FATOR, V_VRLANCTO);

            /* SOMA OS VALORES DOS ENCARGOS AO VALOR DO LANAMENTO */
            FOR C_ENC IN
               (SELECT ROUND(VRITEMENCA, 2) VRITEMENCA, E.IDENCARGO
                 FROM ITEMENCARGO IE, ENCARGO E
                WHERE IE.CDFILIAL   = P_CDFILIAL
                  AND IE.NRLANCTONF = P_NRLANCTONF
                  AND IE.CDENCARGO  = E.CDENCARGO
                  AND IE.VRITEMENCA > 0
                  AND E.IDENCARGO IN ('A','D')
                  AND (DECODE(C_NOTAFISCAL.IDENTRSAIDOP, 'E', CDCTBLENTREN, CDCTBLSAIDEN) IS  NOT NULL))LOOP
               IF C_ENC.IDENCARGO = 'A' THEN
                  V_VRTEMP := V_VRTEMP - (C_ENC.VRITEMENCA * V_FATOR);
               ELSE
                  V_VRTEMP := V_VRTEMP + (C_ENC.VRITEMENCA * V_FATOR);
               END IF;
            END LOOP;

            V_VRLANCTO := V_VRLANCTO + (V_VRTEMP * V_FATOR);

            IF C_NOTAFISCAL.IDOPERDEVONF <> 'S' THEN
               FOR C_RETIRA_IMP IN
                  (SELECT A.CDIMPOSTO, NVL(ROUND(SUM(A.VRIMPOSTONF),2),0) AS VRIMPOSTONFE,
                          A.IDTRATIMPO, A.IDIMPORECUP, A.IDACRPRODENTRNF,
                          MAX(CDCONTATIV) AS CDCONTATIV, MAX(CDCONTPASS)  AS CDCCONTPASS,
                          MAX(CDCONTDESP)  AS CDCONTDESP, MAX(CDCONTIMPO) AS CDCONTIMPO, A.CDFISCALNF
                     FROM (SELECT INF.CDIMPOSTO, VRIMPOSTONF, NVL(IDACRPRODENTRNF, 'N') IDACRPRODENTRNF,
                                  PF.IDCARACFIS, I.IDIMPORECUP, I.IDTRATIMPO, INF.CDFISCALNF,
                                  CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTATIV,
                                  CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTPASS,
                                  CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTDESP,
                                  CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'E', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTIMPO
                             FROM IMPOSTONF INF, IMPOSTO I, NOTAFISCAL NF, PARAMFILIAL PF
                            WHERE NF.CDFILIAL     = P_CDFILIAL
                              AND NF.NRLANCTONF   = P_NRLANCTONF
                              AND PF.CDFILIAL(+)  = NF.CDFILIRECE
                              AND INF.CDFILIAL    = P_CDFILIAL
                              AND INF.NRLANCTONF  = P_NRLANCTONF
                              AND INF.CDIMPOSTO   = I.CDIMPOSTO
                              AND INF.VRIMPOSTONF > 0
                              AND ((IDDIFALIQCUSPRN = 'S' AND INF.IDIMPOSTONF IN ('1','4')) OR (IDDIFALIQCUSPRN = 'N' AND INF.IDIMPOSTONF IN ('1','4')))
                              AND I.IDTIPCONTAB  = 'D'
                              AND I.IDCONTAIMPADIC = 'N') A
                    WHERE ((((A.IDIMPORECUP  = 'S' AND A.CDFISCALNF = '1' AND NVL(A.IDCARACFIS,'C') = 'C') OR (A.IDCARACFIS = 'A')) AND (A.CDCONTATIV IS NOT NULL)) OR
                           ((A.IDIMPORECUP <> 'S') AND (A.CDCONTPASS IS NOT NULL)))
                    GROUP BY A.CDIMPOSTO, A.IDTRATIMPO,  A.IDIMPORECUP, A.IDACRPRODENTRNF, A.CDFISCALNF) LOOP

                  /* CONDICOES REALIZADAS PARA QUE O VALOR DO IMPOSTO SEJA CALCULADO DE ACORDO COM
                     O ANTIGO ONCALCFIELDS DA QUERY ADQIMPOSTONFE */
                  IF ((C_NOTAFISCAL.IDOPERDEVONF = 'S') OR (C_NOTAFISCAL.CDTIPOOPER = '16')) THEN
                     IF C_RETIRA_IMP.IDTRATIMPO = 'V' THEN
                        V_VRTEMP := ROUND(C_RETIRA_IMP.VRIMPOSTONFE, 2);
                     ELSE
                        V_VRTEMP := - ROUND(C_RETIRA_IMP.VRIMPOSTONFE, 2);
                     END IF;

                     IF (C_RETIRA_IMP.IDTRATIMPO = 'A' AND C_RETIRA_IMP.IDACRPRODENTRNF = 'N') THEN
                        V_VRTEMP  := ROUND(V_VRTEMP, 2);
                     END IF;
                  ELSE
                     IF C_RETIRA_IMP.IDTRATIMPO = 'V' AND C_RETIRA_IMP.IDIMPORECUP = 'S' THEN
                        V_VRTEMP := - ROUND(C_RETIRA_IMP.VRIMPOSTONFE, 2);
                     ELSE
                        V_VRTEMP := ROUND(C_RETIRA_IMP.VRIMPOSTONFE, 2);
                     END IF;

                     IF C_RETIRA_IMP.IDTRATIMPO = 'A' AND C_RETIRA_IMP.IDACRPRODENTRNF = 'N' THEN
                        V_VRTEMP := - ROUND(V_VRTEMP, 2);
                     ELSE
                        V_VRTEMP := ROUND(V_VRTEMP, 2);
                     END IF;
                  END IF;
                  V_VRLANCTO := V_VRLANCTO + (V_VRTEMP * V_FATOR);
               END LOOP;
            END IF;

            /* CONTABILIZA OS VALORES DOS PRODUTOS */
            CTB_DFE_PRODUTO(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_FATOR, V_VRLANCTO, V_VRINDUSTRIA);

            CTB_DFE_IMPOSTO(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_FATOR, V_VRLANCTO, 1);

            CTB_DFE_HISTORICO(P_CDFILIAL, P_NRLANCTONF, V_CDHISTPADR, V_DSHISTLANC);

            CONTABILIZA_ENCARGO(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIRECE, C_NOTAFISCAL.NRLANCTONF,
                                C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                                C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTENTRSAID);

            CONTABILIZA_DESCONTO(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIRECE, C_NOTAFISCAL.NRLANCTONF,
                                 C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                                 C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTENTRSAID);

            DESPESA_DIFALIQUOTA(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIRECE, C_NOTAFISCAL.NRLANCTONF,
                                C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                                C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTENTRSAID);

         ELSE
            /* NESTA PROCEDURE  FEITA AS PARTDAS DE CRDITO PARA OS IMPOSTOS DE BONIFICAO.
               AS PARTIDAS DE DBITO SO FEITAS NA PROCEDURE DE IMPOSTO, PASSANDO O FATOR PARA ALTERAO DE SINAL */
            CTB_DFE_IMPBONIFIC(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_FATOR, V_VRLANCTO);
         END IF;

         /*VERIFICACAO DE ERROS NAS PARTIDAS E NO LANCAMENTO CONTABIL*/
         VERIFICA_ERRO_CTB(V_NRLANCTEMP, C_NOTAFISCAL.CDEMPRESA);
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM ERROCTBTEMP
             WHERE NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;

         /* CASO NO TENHA GERADO ERRO NA ROTINA DE CONTABILIZAO, O NMERO DE LANAMENTO  GRAVADO NA NOTAFISCAL. */
         IF V_COUNT = 0 THEN
            IF C_NOTAFISCAL.NRLANCTBDOC IS NULL THEN
               UPDATE NOTAFISCAL
                  SET NRLANCTBDOC = V_NRLANCTEMP
                WHERE CDFILIAL    = P_CDFILIAL
                  AND NRLANCTONF  = P_NRLANCTONF;
            END IF;
         END IF;
      ELSE
         DELETE
           FROM LANCCONT
          WHERE NRLANCCONT = C_NOTAFISCAL.NRLANCTBDOC
            AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;                              
      END IF;
      P_NRLANCCTB := V_NRLANCTEMP;

      /* TRMINO DAS PARTIDAS DO LANCAMENTO CONTABIL*/
       /* GRAVA O LANAMENTO TEMPORRIO SE NO CONTER ERRO NA LANCCONT E LANCCUSTO, APS GRAVAR IR APAGAR O LANCAMENTO TEMPORRIO */
      GRAVA_LANC_CTB(V_NRLANCTEMP, C_NOTAFISCAL.CDEMPRESA);
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CTB_DFE_CONTAB','6.49.040'); END;
/

