/*+
|                                               |
|          Procedure: ETL_FATUINVOICE           |
|                                               |
| Rotina para insero de dados de BI -         |
| valores de Faturamento / Comissaria,          |
| para visualizao no CVI.                     |
|                                               |
|            Tabelas Preenchidas:               |
|                                               |
|       1: ETLFAUINVOICE - Faturamento          |
|                                               |
|     Criao:                                  |
|     23/10/2007 - Breno de Assis e Castro      |
|                                               |
+*/


create or replace
PROCEDURE ETL_FATUINVOICE
   ( P_DTINI  IN DATE,
     P_DTFINA IN DATE )
IS
   V_DTFIM                  DATE;
   V_DTINI                  DATE;

   V_MES                    VARCHAR2(02);
   V_ANO                    VARCHAR2(04);

   V_ZERO                   NUMBER := 0;
   V_CONT                   NUMBER := 0;

BEGIN

   V_MES  :=  TO_CHAR(P_DTINI,'MM');
   V_ANO  :=  TO_CHAR(P_DTINI,'YYYY');

   DELETE FROM ETLFATUINVOICE
   WHERE  (ANETLFAC > V_ANO)
   OR     (ANETLFAC = V_ANO AND MEETLFAC >= V_MES);
   COMMIT;
 
   V_DTINI := ADD_MONTHS(LAST_DAY(P_DTINI) + 1, -1) ;

   WHILE (V_DTINI <= P_DTFINA)  /*LOOP ENTRE DATA INICIO E DATA FIM*/
   LOOP
      V_DTFIM := LAST_DAY(V_DTINI);
      V_MES :=  TO_CHAR(V_DTFIM,'MM');
      V_ANO :=  TO_CHAR(V_DTFIM,'YYYY');
--------------------------------------------------------------------------------
          ----------CURSOR PARA LOOP DE FATURAMENTO POR FILIAL----------
      FOR C_ETL IN
         (SELECT
                F.CDFILIAL,        F.CDAREAGERE,        F.CDSEGMER,
                F.CDEMPRESA,       F.CDSUPERVISOR,      F.DTINICIALBI,
                F.CDGOPERAC,       EN.CDPAIS          
          FROM
                FILIAL F , ENDEFILI EN
          WHERE
                F.CDFILIAL = EN.CDFILIAL
          AND   F.IDFUNCAOFILI IS NULL
          AND   F.DTINICIALBI IS NOT NULL
         )
      LOOP
         IF ADD_MONTHS(LAST_DAY(C_ETL.DTINICIALBI) + 1, -1) <= V_DTINI THEN
--------------------------------------------------------------------------------
            ---------CURSOR PARA LOOP DE FATURAMENTO POR CLIENTE----------
            FOR C_CLIENTE IN
              (SELECT
                     NF.CDCLIENTE
               FROM
                     NOTAFISCAL NF, COMPANHIA CO, CIAFILI CF
               WHERE
                     NF.CDEMPRESA  = C_ETL.CDEMPRESA
               AND   CF.CDCIA      = CO.CDCIA
               AND   NF.CDCLIENTE  = CO.CDCLIENTE
               AND   NF.CDFILIAL   = C_ETL.CDFILIAL
               AND   NF.CDCLIENTE IS NOT NULL
               AND   NF.IDSTATUS  IN ('A','E')
               AND   NVL(C_ETL.CDFILIAL, CF.CDFILIAL) = CF.CDFILIAL
               AND   NF.DTEMISSAO BETWEEN V_DTINI AND V_DTFIM
               GROUP BY
                     NF.CDCLIENTE
              )
            LOOP
--------------------------------------------------------------------------------
        ----------CURSOR PARA FILTRAR E CALCULAR DADOS DE FATURAMENTO----------
               FOR C_VOICE IN
                 (SELECT
                         NF.CDFILIAL,
                         CL.CDCLIENTE,
                        (SUM ( NVL( RE.VRMERCADORIALI ,0 ))+   SUM(NVL(HD.VRHANDLINGLI,0))+ SUM(NVL(TA.VRTXAERO,0))+ SUM(NVL(AC.VRITEMENCAAC,0))- SUM(NVL(IE.VRITEMENCA,0))) AS VRFATURAMENTO,/*VALOR TOTAL DE FATURAMENTO*/
                         SUM ( NVL( RE.VRMERCADORIALI ,0 ))    AS VRMERCADORIALI, /*VALOR DAS MERCADORIAS  */
                         SUM ( NVL( HD.VRHANDLINGLI   ,0 ))    AS VRHANDLINGLI,   /*VALOR DOS SERVIOS  */
                         SUM ( NVL( TA.VRTXAERO       ,0 ))    AS VRTXAERO,       /*VALOR TAXA AEROPORTO  */
                         SUM ( NVL( IE.VRITEMENCA     ,0 ))    AS VRITEMENCA ,    /*VALOR DE ENCARGOS DESCONTOS*/
                         SUM ( NVL( AC.VRITEMENCAAC   ,0 ))    AS VRITEMENCAAC,   /*VALOR DE ENCARGOS ACRESCIMOS*/
                        (SUM ( NVL( VRIMPOSCOFINS     ,0 ))+   SUM(NVL(VRIMPOSPIS,0))+ SUM (NVL(VRIMPOSICMS,0))+ SUM(NVL(VRIMPOSISS,0 ))+  SUM(NVL(TA.VRTXAERO,0))) AS VRIMPOSTOS /*VALOR DE SOMA DOS IMPOSTOS*/

                  FROM CLIENTE CL,

                       ( SELECT
                                NF.CDCLIENTE,      NF.NRLANCTONF,      CLI.NMRAZSOCCLIE,
                                NF.DTENTRSAID,     NF.CDFILIAL
                         FROM
                               CLIENTE CLI, NOTAFISCAL NF
                         WHERE
                               CLI.CDCLIENTE = NF.CDCLIENTE
                         AND   NF.CDFILIAL   = C_ETL.CDFILIAL
                         AND   NF.CDCLIENTE  = C_CLIENTE.CDCLIENTE
                         AND   NF.CDCLIENTE IS NOT NULL
                         AND   NF.IDSTATUS  IN ('A','E')
                         AND   NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM
                       )NF,

  --------------------------------------------------------------------------------
                        ----------VALOR  MERCADORIA----------
                       ( SELECT
                               NOTAFISCAL,    NRLANCTONF,   NOMECLIENTE,
                               DATANOTA,      CDCLIENTE,
                               SUM(VRMERCADORIA)    VRMERCADORIA,
                               SUM(VRMERCADORIALI)  VRMERCADORIALI
                         FROM
                             ( SELECT DISTINCT
                                      IP.CDFILIAL,
                                      IP.NRSEQITPROD,
                                      IP.NRLANCTONF      NRLANCTONF,
                                      CLI.NMRAZSOCCLIE   NOMECLIENTE,
                                      NF.NRLANCTONF      NOTAFISCAL,
                                      NF.DTENTRSAID      DATANOTA,
                                      CLI.CDCLIENTE      CDCLIENTE,

                                     (IP.VRBRUTITEM)     VRMERCADORIA,
                                     (IP.VRTOTAITEM)     VRMERCADORIALI
                               FROM
                                      ITEMPRODUTO IP,   PARAMCOMIS PG,
                                      NOTAFISCAL NF,    CLIENTE CLI
                               WHERE
                                     ( IP.CDFILIAL    =  C_ETL.CDFILIAL     )
                               AND   ( NF.CDCLIENTE   =  C_CLIENTE.CDCLIENTE)
                               AND   ( NF.CDCLIENTE   =  CLI.CDCLIENTE      )
                               AND   ( IP.CDFILIAL    =  NF.CDFILIAL        )
                               AND   ( NF.CDFILIAL    =  PG.CDFILIAL        )
                               AND   ( IP.NRLANCTONF  =  NF.NRLANCTONF      )
                               AND   ( IP.CDFILIAL    =  PG.CDFILIAL        )
                               AND   ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                               AND   ((PG.CDPRODUTO  IS NULL) OR (IP.CDPRODUTO <> PG.CDPRODUTO))
                               AND   ((PG.CDPRAEROIN IS NULL) OR (IP.CDPRODUTO <> PG.CDPRAEROIN))
                               AND   NOT EXISTS ( SELECT
                                                       NRSEQITPROD
                                                  FROM
                                                       ITEMPROIMPOS II
                                                  WHERE
                                                       II.CDFILIAL    = IP.CDFILIAL
                                                  AND  II.NRLANCTONF  = IP.NRLANCTONF
                                                  AND  II.NRSEQITPROD = IP.NRSEQITPROD
                                                  AND  II.CDIMPOSTO   = '31'
                                                )
                             )
                         GROUP BY
                              CDCLIENTE,
                              NOTAFISCAL,
                              NRLANCTONF,
                              NOMECLIENTE,
                              DATANOTA
                       ) RE,

  --------------------------------------------------------------------------------
                       --------VALOR SERVIOS--------
                       ( SELECT DISTINCT
                               IP.NRLANCTONF,       NF.CDCLIENTE,
                               CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                               SUM(IP.VRBRUTITEM) VRHANDLING,
                               SUM(IP.VRTOTAITEM) VRHANDLINGLI
                         FROM
                               ITEMPRODUTO IP,   PARAMCOMIS PG,
                               ITEMPROIMPOS II,  NOTAFISCAL NF,
                               CLIENTE CLI
                         WHERE
                               ( IP.CDFILIAL     =  C_ETL.CDFILIAL     )
                         AND   ( NF.CDCLIENTE    =  C_CLIENTE.CDCLIENTE)
                         AND   ( NF.CDCLIENTE    =  CLI.CDCLIENTE      )
                         AND   ( IP.CDFILIAL     =  NF.CDFILIAL        )
                         AND   ( NF.CDFILIAL     =  PG.CDFILIAL        )
                         AND   ( PG.CDFILIAL     =  II.CDFILIAL        )
                         AND   ( II.NRLANCTONF   =  IP.NRLANCTONF      )
                         AND   ( II.NRLANCTONF   =  NF.NRLANCTONF      )
                         AND   ( IP.NRLANCTONF   =  NF.NRLANCTONF      )
                         AND   ( II.NRSEQITPROD  =  IP.NRSEQITPROD     )
                         AND   ( II.CDIMPOSTO    =  '31'               )
                         AND   ( NF.CDCLIENTE IS NOT NULL              )
                         AND   ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         AND   ((PG.CDPRODUTO  IS NULL) OR (IP.CDPRODUTO <> PG.CDPRODUTO))
                         AND   ((PG.CDPRAEROIN IS NULL) OR (IP.CDPRODUTO <> PG.CDPRAEROIN))
                         GROUP BY
                               NF.CDCLIENTE,
                               IP.NRLANCTONF,
                               CLI.NMRAZSOCCLIE,
                               NF.DTENTRSAID
                       ) HD,


  --------------------------------------------------------------------------------
                       ------------VALOR TAXA AEROPORTO--------------
                       ( SELECT
                              IP.CDFILIAL,          NF.CDCLIENTE,
                              CLI.NMRAZSOCCLIE,     NF.NRLANCTONF,
                              SUM(IP.VRTOTAITEM) VRTXAERO,
                              SUM(IP.VRBRUTITEM) VRTXAEROB
                         FROM
                              ITEMPRODUTO IP, PARAMCOMIS PG, NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                               ( IP.CDFILIAL     =  PG.CDFILIAL            )
                         AND   ( IP.CDFILIAL     =  C_ETL.CDFILIAL         )
                         AND   ( NF.CDCLIENTE    =  C_CLIENTE.CDCLIENTE    )
                         AND   ( NF.CDCLIENTE    =  CLI.CDCLIENTE          )
                         AND   ( NF.NRLANCTONF   =  IP.NRLANCTONF          )
                         AND   ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM )
                         AND   (( IP.CDPRODUTO = PG.CDPRODUTO ) OR ( IP.CDPRODUTO = PG.CDPRAEROIN ))
                         GROUP BY
                                  IP.CDFILIAL,
                                  NF.CDCLIENTE,
                                  CLI.NMRAZSOCCLIE,
                                  NF.NRLANCTONF
                       ) TA,

  --------------------------------------------------------------------------------
                       ------------VALOR DE ENCARGOS-------------
                       ( SELECT
                                IE.NRLANCTONF,        NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,     NF.DTENTRSAID,
                                SUM(IE.VRITEMENCA) VRITEMENCA
                         FROM
                                ITEMENCARGO IE, ENCARGO EN,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IE.CDFILIAL     =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE    =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE      )
                         AND  ( IE.NRLANCTONF   =  NF.NRLANCTONF      )
                         AND  ( IE.CDENCARGO    = EN.CDENCARGO        )
                         AND  ( IE.CDENCARGO BETWEEN '06' AND '13'    )
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         GROUP BY
                                NF.CDCLIENTE,
                                IE.NRLANCTONF,
                                CLI.NMRAZSOCCLIE,
                                NF.DTENTRSAID
                       ) IE,

                       ( SELECT
                                IE.NRLANCTONF,      NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,   NF.DTENTRSAID,
                                SUM(IE.VRITEMENCA) VRITEMENCAAC
                         FROM
                                ITEMENCARGO IE, ENCARGO EN,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IE.CDFILIAL     =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE    =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE      )
                         AND  ( IE.NRLANCTONF   =  NF.NRLANCTONF      )
                         AND  ( IE.CDENCARGO    = EN.CDENCARGO        )
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         AND  ( IE.CDENCARGO   IN ('01','03','04','05','14','15'))
                         GROUP BY
                                NF.CDCLIENTE,
                                IE.NRLANCTONF,
                                CLI.NMRAZSOCCLIE,
                                NF.DTENTRSAID
                       ) AC,

  --------------------------------------------------------------------------------
                       -------VALOR DE IMPOSTO NOTA FISCAL---------
                       ( SELECT
                                IF.NRLANCTONF,       NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                SUM(IF.VRTOTIMPONF) VRIMPOSICMS
                         FROM
                                IMPOSNFTOT IF,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IF.CDFILIAL   =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE  =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE  =  CLI.CDCLIENTE      )
                         AND  ( IF.NRLANCTONF =  NF.NRLANCTONF      )
                         AND  ( IF.CDIMPOSTO   IN  ('00'))
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         GROUP BY
                                NF.CDCLIENTE,
                                IF.NRLANCTONF,
                                NF.DTENTRSAID,
                                CLI.NMRAZSOCCLIE
                       ) IC,
                       ( SELECT
                                IF.NRLANCTONF,       NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                SUM(IF.VRTOTIMPONF) VRIMPOSISS
                         FROM
                                IMPOSNFTOT IF,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IF.CDFILIAL   =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE  =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE  =  CLI.CDCLIENTE      )
                         AND  ( IF.NRLANCTONF =  NF.NRLANCTONF      )
                         AND  ( IF.CDIMPOSTO   IN  ('31'))
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         GROUP BY
                                NF.CDCLIENTE,
                                IF.NRLANCTONF,
                                NF.DTENTRSAID,
                                CLI.NMRAZSOCCLIE
                       ) SS ,
                       ( SELECT
                                IF.NRLANCTONF,       NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                SUM(IF.VRTOTIMPONF) VRIMPOSPIS
                         FROM
                                IMPOSNFTOT IF,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IF.CDFILIAL   =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE  =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE  =  CLI.CDCLIENTE      )
                         AND  ( IF.NRLANCTONF =  NF.NRLANCTONF      )
                         AND  ( IF.CDIMPOSTO   IN  ('50'))
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         GROUP BY
                                NF.CDCLIENTE,
                                IF.NRLANCTONF,
                                NF.DTENTRSAID,
                                CLI.NMRAZSOCCLIE
                       ) PI,
                       ( SELECT
                                IF.NRLANCTONF,       NF.CDCLIENTE,
                                CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                SUM(IF.VRTOTIMPONF) VRIMPOSCOFINS
                         FROM
                                IMPOSNFTOT IF,
                                NOTAFISCAL NF, CLIENTE CLI
                         WHERE
                              ( IF.CDFILIAL   =  C_ETL.CDFILIAL     )
                         AND  ( NF.CDCLIENTE  =  C_CLIENTE.CDCLIENTE)
                         AND  ( NF.CDCLIENTE  =  CLI.CDCLIENTE      )
                         AND  ( IF.NRLANCTONF =  NF.NRLANCTONF      )
                         AND  ( IF.CDIMPOSTO   IN  ('60'))
                         AND  ( NF.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                         GROUP BY
                                NF.CDCLIENTE,
                                IF.NRLANCTONF,
                                NF.DTENTRSAID,
                                CLI.NMRAZSOCCLIE
                       ) CF

  --------------------------------------------------------------------------------
                  WHERE
                        ( NF.CDFILIAL    =  C_ETL.CDFILIAL      )
                  AND   ( CL.CDCLIENTE   =  C_CLIENTE.CDCLIENTE )
                  AND   ( NF.NRLANCTONF  =  RE.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  HD.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  TA.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  AC.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  IE.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  IC.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  SS.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  PI.NRLANCTONF(+)    )
                  AND   ( NF.NRLANCTONF  =  CF.NRLANCTONF(+)    )
                  GROUP BY
                          NF.CDFILIAL,
                          CL.CDCLIENTE
               )
               LOOP
-------------------------------------------------------------------------------


                      /*INSERO DE DADOS NA TABELA DE RESULTADOS DO FATURAMENTO*/
                  INSERT INTO  ETLFATUINVOICE
                    (  CDFILIAL,        CDEMPRESA ,      CDAREAGERE ,
                       CDPAIS,          CDCLIENTE,       ANETLFAC,
                       MEETLFAC,        VRFATUETLFAC,    VRMERCETLFAC,
                       VRSERVETLFAC,    VRTXARETLFAC,    VRDESCETLFAC,
                       VRACREETLFAC,    VRIMPOETLFAC
                    ) 
                  VALUES
                    (  C_ETL.CDFILIAL,          C_ETL.CDEMPRESA,          C_ETL.CDAREAGERE,
                       C_ETL.CDPAIS,            C_CLIENTE.CDCLIENTE,      V_ANO,
                       V_MES,                   C_VOICE.VRFATURAMENTO,    C_VOICE.VRMERCADORIALI,
                       C_VOICE.VRHANDLINGLI,    C_VOICE.VRTXAERO,         C_VOICE.VRITEMENCA,
                       C_VOICE.VRITEMENCAAC,    C_VOICE.VRIMPOSTOS
                    );


                  V_CONT := V_CONT + 1;
                  IF V_CONT >= 500 THEN /*COMMIT A CADA 500 INSERES*/
                      COMMIT;
                      V_CONT := 0;
                  END IF; /*Fim IF PARA COMMIT*/

               END LOOP; /*FIM LOOP C_VOICE*/

            END LOOP;/*END LOOP CLIENTE*/

         END IF; /*End fim if date*/

      END LOOP; /*Fim loop for C_ETL*/

      COMMIT;
      V_DTINI := ADD_MONTHS (V_DTINI, 1);
   END LOOP;/*Fim loop while data inicial / data final*/
END;/*Fim ETL_FATUINVOICE*/

/

-- VERSAO --
BEGIN ATUVEROBJ('ETL_FATUINVOICE','5.05.019'); END;
/
