import {
  Modal, ModalService, Grid, DialogService, Form, Select, Date as ZdDate, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam, I18n,
  Loader, Http, dayjs as moment,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00609ImportacaoTpPrato {
  public utilities = Loader.getInstance('Utilities');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public modalImportacao!: Modal;
  
  public selectedLine: IDictionary = {};
  
  // Funçoes search modal Importacao
  public async createPeriodoModal() {
    this.modalImportacao = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/gridPeriodoImportacao' });
  }
  
  public openModalImportacao() {
    this.modalImportacao.show();
  }
  
  public destroyPeriodoModal() {
    this.modalImportacao.destroy();
  }
  
  public closeModal() {
    this.modalImportacao.hide();
  }

  // Funções da importação por tipo de prato.
  
  public openModalSearch() {
    const tipoCard = Metadata.getInstance<Form>('formFilterImportTpPrato').value.TIPOCARD_IMPORTPRAT;
    const central = Metadata.getInstance<Form>('formFilterImportTpPrato').value.CENTRAL;
    if (tipoCard) {
      this.openModalImportacao();
      this.utilities.setCentral(central);
      this.utilities.setCdTipoCard(tipoCard);
      this.utilities.setFilterModalPeriodoImport();
    } else {
      this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    }
  }
  
  public reloadModalPeriodo() {
    const central = Metadata.getInstance<Form>('formFilterImportTpPrato').value.CENTRAL;
    const tipoCard = Metadata.getInstance<Form>('formFilterImportTpPrato').value.TIPOCARD_IMPORTPRAT;
    this.utilities.setFilterModalPeriodoCentralModal(central, tipoCard);
  }
  
  public setPeriodoParams({ row }: IDictionary) {
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const dtDe = Metadata.getInstance<ZdDate>('DTDE');
    const dtAte = Metadata.getInstance<ZdDate>('DTATE');
    periodoDe.value = row.DTINICARD;
    periodoAte.value = row.DTFINCARD;
    dtDe.value = row.DTINICARD;
    dtAte.value = row.DTFINCARD;
    this.modalImportacao.hide();
    this.setFilterGridImportacao();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TIPOCARD_IMPORTPRAT');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const dtDe = Metadata.getInstance<ZdDate>('DTDE');
    const dtAte = Metadata.getInstance<ZdDate>('DTATE');

    if (component.name === 'CENTRAL') {
      cardapio.setValue('');
      servico.setValue('');
      periodoDe.value = '';
      periodoAte.value = '';
      dtDe.value = '';
      dtAte.value = '';
    } else if (component.name === 'TIPOCARD_IMPORTPRAT') {
      servico.setValue('');
      periodoDe.value = '';
      periodoAte.value = '';
      dtDe.value = '';
      dtAte.value = '';
    }

    const gridImportacaoTpPrato = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    gridImportacaoTpPrato.datasource.data = [];
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', form.value.CENTRAL);
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: form.value.TIPOCARD_IMPORTPRAT ? form.value.TIPOCARD_IMPORTPRAT : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: form.value.CENTRAL,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public validateDate(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const dtDe = row.DTDE;
    const dtAte = row.DTATE;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    const concatDate = moment(periodoDe, 'DD/MM/YYYY').format(
      this.utilities.getFormatLocale(),
    ) + String(' a ')
      + moment(periodoAte, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    
    if (this.utilities.dateIsValid(periodoDe, periodoAte) && this.utilities.dateIsValid(dtDe, dtAte)) {
      if (this.utilities.dateIsValid(periodoDe, dtDe) && this.utilities.dateIsValid(dtAte, periodoAte)) valid = true;
      else {
        this.utilities.showAlert(
          'alertError',
          I18n.translate('pme::dtCompreendidaPerCardImportacao', { PARAM1: concatDate }),
          'error',
        );
      }
    } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    return valid;
  }
  
  public validateDateIncluirCard(form: Form) {
    let valid = false;
    const dtCardIniPeriodo = this.elaboracaoCardapio.getFilterData.initialDate;
    const dtCardFinPeriodo = this.elaboracaoCardapio.getFilterData.finalDate;
    const concatDate = moment(dtCardIniPeriodo, 'DD/MM/YYYY').format(this.utilities.getFormatLocale())
      + String(' a ') + moment(dtCardFinPeriodo, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    const row: IDictionary = form.value;
    const incluirCard = moment(row.INCLUIRCARD, 'DD/MM/YYYY').format('DD/MM/YYYY');
    
    if (incluirCard < dtCardIniPeriodo || incluirCard > dtCardFinPeriodo) {
      this.utilities.showAlert(
        'alertError',
        I18n.translate('pme::dtCompreendidaPerCardImportacao', { PARAM1: concatDate }),
        'error',
      );
    } else valid = true;
    return valid;
  }
  
  public validDataChange() {
    const gridImportacaoTpPrato = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    const format = 'DD/MM/YYYY';
    const periodoDe = moment(Metadata.getInstance<ZdDate>('PERIODODE').value, format);
    const periodoAte = moment(Metadata.getInstance<ZdDate>('PERIODOATE').value, format);
    const dtDe = moment(Metadata.getInstance<ZdDate>('DTDE').value, format);
    const dtAte = moment(Metadata.getInstance<ZdDate>('DTATE').value, format);
    
    if ((dtDe < periodoDe || dtDe > periodoAte) || (dtAte < dtDe || dtAte > periodoAte)) {
      this.utilities.showAlert('foraDoPeriodo', 'pme::dataForaDoPeriodoSelecionado', 'error');
      gridImportacaoTpPrato.datasource.data = [];
    } else {
      this.setFilterGridImportacao();
    }
  }
  
  public setFilterGridImportacao() {
    const gridImportacaoTpPrato = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const central = Metadata.getInstance<Select>('CENTRAL');
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD_IMPORTPRAT');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const dtDe = Metadata.getInstance<ZdDate>('DTDE');
    const dtAte = Metadata.getInstance<ZdDate>('DTATE');
    const row = form.value;
    
    if (dtDe.value === '' || dtAte.value === '') {
      periodoDe.value = '';
      periodoAte.value = '';
    }
    
    if (servico.value) {
      if ((periodoDe.value !== '') && (periodoAte.value !== '')) {
        if (this.validateDate(form)) {
          if (this.validateDateIncluirCard(form)) {
            Http.post('configCard/cardapioImportacao', { requestType: 'Row', row }).then((response) => {
              const result = response.data.dataset.result;
              if (result.length !== 0) {
                gridImportacaoTpPrato.datasource.loading = true;
                const filter = [
                  {
                    name: 'P_DTFINCARDPME',
                    operator: '=',
                    value: periodoAte.value,
                  },
                  {
                    name: 'P_NRCARDPME',
                    operator: '=',
                    value: result[0].NRCARDPME,
                  },
                  {
                    name: 'P_CDFILCENTPME',
                    operator: '=',
                    value: central.value,
                  },
                  {
                    name: 'P_CDSERVFILI',
                    operator: '=',
                    value: servico.value,
                  },
                  {
                    name: 'P_DTINICARDPME',
                    operator: '=',
                    value: periodoDe.value,
                  },
                  {
                    name: 'P_CDTIPOCARD',
                    operator: '=',
                    value: tipoCard.value,
                  },
                ];
                
                gridImportacaoTpPrato.datasource.filter = filter;
                gridImportacaoTpPrato.datasource.get();
              } else gridImportacaoTpPrato.datasource.data = [];
            });
          }
        } else gridImportacaoTpPrato.datasource.data = [];
      } else {
        form.validate();
        this.utilities.showAlert('alert', 'pme::periodoObrig', 'error');
        gridImportacaoTpPrato.datasource.data = [];
      }
    } else gridImportacaoTpPrato.datasource.data = [];
  }
  
  public openDialog() {
    const grid = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    this.selectedLine = grid.selectedRows;
    if (this.selectedLine.length === 0) {
      form.validate();
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    } else if (this.validateDateIncluirCard(form)) {
      // const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
      DialogService.show({
        name: 'pme_warningImportacao',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::acrescimoTpPrato',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.saveImportTpPrato(this.selectedLine) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public saveImportTpPrato(checkedRows: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const rowForm = form.value;
    // const button = Metadata.getInstance<Button>('yes');
    
    checkedRows.forEach((item: IDictionary) => {
      item.CENTRALDESTINO = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      item.SERVICODESTINO = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
      item.TIPOCARDDESTINO = this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD;
      item.NRCARDPMEDESTINO = this.elaboracaoCardapio.getFilterData.NRCARDPME;
      item.CDTIPOCARD = rowForm.TIPOCARD_IMPORTPRAT;
      item.DTDE = rowForm.DTDE;
      item.DTATE = rowForm.DTATE;
      item.INCLUIRCARD = rowForm.INCLUIRCARD;
      item.DATAFINALCARDAPIO = this.elaboracaoCardapio.getFilterData.finalDate;
    });
    
    LoadingService.show(' ');
    Http.post('/configCard/saveImportTpPrato', { checkedRows }).then(() => {
      this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
    }).finally(() => {
      DialogService.hide();
      LoadingService.hide();
    });
  }
  
  public setDataPosicionada() {
    const dtIncluir = Metadata.getInstance<ZdDate>('INCLUIRCARD');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    dtIncluir.value = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
  }
}
