import {
  Loader, Http, IEventParam, Metadata, IDictionary, dayjs as moment,
  I18n,
} from '@zeedhi/core';
import {
  Select, Form, Date as ZdDate, GridEditable, LoadingService, SelectMultiple, IComponentRender, DialogService,
  TextInput,
} from '@zeedhi/common';

export class Pme05900AssistenteIntegracaoCardapio {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public crudController = Loader.getInstance('CrudController');
  
  public editable = false;

  // public validaCentralIntegracaoCard() {
  //   const row = {
  //     CDFILCENTPME: this.appController.getCdCentral(),
  //   };
  //   LoadingService.show(' ');
  //   Http.post('/validaCentralIntegracaoCard', { requestType: 'Row', row }).then((response) => {
  //     const result = response.data.dataset.result;
  //     if (result.length > 0) {
  //       Http.post('/dataProcessamentoCentral', { requestType: 'Row', row }).then((value) => {
  //         const dataProcessamentoCentral = value.data.dataset.result[0];
  //         if (dataProcessamentoCentral < this.utilities.dtProcessFormatada) {
  //           this.utilities.showAlert('alertError', 'pme::dataProcessamento', 'red');
  //         }
  //       });
  //     } else this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'red');
  //   }).finally(() => {
  //     LoadingService.hide();
  //   });
  // }
  
  public setCentral({ component }: IEventParam<Select>) {
    let campo: Select;
    if (component.name === 'CENTRAL_ORIGEM') {
      campo = Metadata.getInstance('CENTRAL_ORIGEM');
    } else {
      campo = Metadata.getInstance('CENTRAL_DESTINO');
    }
    
    campo.setValue(this.appController.getCdCentral());
  }
  
  public clearDependentField({ component }: IDictionary) {
    const grupTpCard: Select = Metadata.getInstance('CDGRUPTIPOCARD');
    const tpCard: SelectMultiple = Metadata.getInstance('TPCARD');
    const servico: SelectMultiple = Metadata.getInstance('CDSERVFILI');

    if (component.name === 'CENTRAL_ORIGEM') {
      grupTpCard.setValue(null);
      tpCard.setValue(null);
      servico.setValue(null);
    } else if (component.name === 'CDGRUPTIPOCARD') {
      tpCard.setValue(null);
      servico.setValue(null);
    } else if (component.name === 'TPCARD' || component.name === 'GRUPOSERVICO') {
      servico.setValue(null);
    } else if (component.name === 'DTDE') {
      const finalDate: ZdDate = Metadata.getInstance('DTATE');
      finalDate.value = null;
    }
    
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    grid.datasource.data = [];
    grid.datasource.filter = [];
  }
  
  public filtraGrupTipoCard({ component }: IEventParam<Select>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public filtraTpCard({ component }: IEventParam<SelectMultiple>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const grupoTpCard = formOrigem.value.CDGRUPTIPOCARD;
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: grupoTpCard === null ? 'T' : grupoTpCard,
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public dataAtual({ component }: IEventParam<ZdDate>) {
    component.value = moment().format('DD/MM/YYYY');
  }
  
  public setFilterService({ component }: IEventParam<SelectMultiple>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const grupoTpCard = formOrigem.value.CDGRUPTIPOCARD;
    const tipoCard = Metadata.getInstance<SelectMultiple>('TPCARD').value;
    const grupoServ = formOrigem.value.GRUPOSERVICO;
    
    const conditions = [
      {
        name: 'P_LISTATPCARD',
        operator: '=',
        value: tipoCard || null,
      },
      {
        name: 'P_TODOSTPCARD',
        operator: '=',
        value: tipoCard.length > 0 ? tipoCard.join('_') : 'T',
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: grupoTpCard || null,
      },
      {
        name: 'P_TGRUPTIPOCARD',
        operator: '=',
        value: grupoTpCard !== null ? grupoTpCard : 'T',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDGRUPSERV',
        operator: '=',
        value: grupoServ !== null ? grupoServ.CDGRUPSERV : null,
      },
      {
        name: 'P_TCDGRUPSERV',
        operator: '=',
        value: grupoServ !== null ? grupoServ.CDGRUPSERV : 'T',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public filtraGridDestino() {
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    
    const cdServFili = (formOrigem.value.CDSERVFILI.length > 0) ? formOrigem.value.CDSERVFILI.join('_') : 'T';
    
    let cdGrupTipoCard = formOrigem.value.CDGRUPTIPOCARD;
    if (cdGrupTipoCard === null || cdGrupTipoCard === undefined) cdGrupTipoCard = 'T';
    
    const cdTipoCard = (formOrigem.value.TPCARD.length > 0) ? formOrigem.value.TPCARD.join('_') : 'T';
    
    const conditions = [
      {
        name: 'P_DATAFIN',
        operator: '=',
        value: formOrigem.value.DTATE,
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: formOrigem.value.CENTRAL_ORIGEM,
      },
      {
        name: 'P_CDSERVFILI',
        operator: '=',
        value: cdServFili,
      },
      {
        name: 'P_DATAINI',
        operator: '=',
        value: formOrigem.value.DTDE,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: cdGrupTipoCard,
      },
      {
        name: 'P_CDTIPOCARD',
        operator: '=',
        value: cdTipoCard,
      },
      {
        name: 'P_CDFILCENTPME_DEST',
        operator: '=',
        value: formCentralDest.value.CENTRAL_DESTINO,
      },
    ];
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
    /* .then((response: any) => {
      const rows = response || []; // Certifique-se de que response.data seja um array

      rows.forEach((row: any) => {
        if (formOrigem.value.CENTRAL_ORIGEM === formCentralDest.value.CENTRAL_DESTINO) {
          row.TPCARD_DESTINO = row.CDTIPOCARD;
          row.CDSERVFILI_DESTINO = row.CDSERVFILI;
        }

        // const component = row.DTINIPERI;
        // if (component) {
        //   const diaSemana = this.returnDiaSemana(component);
        //   row.DIASEMANA = diaSemana;
        // }
      });

      grid.datasource.updateData(rows);
    }); */
  }
  
  public askReloadButton() {
    DialogService.show({
      name: 'reload',
      type: 'warning',
      title: 'pme::atencao',
      text: 'pme::reloadAssistenteIntegracao',
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'pme::sim',
          events: {
            click: () => {
              this.reloadGridDestino();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
  
  public reloadGridDestino() {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const dataDe = formOrigem.value.DTDE;
    const dataAte = formOrigem.value.DTATE;
    
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    grid.datasource.data = [];
    grid.datasource.filter = [];
    
    if (formOrigem.validate() && formCentralDest.validate()) {
      if (this.utilities.dateIsValid(dataDe, dataAte)) {
        this.setGridDestino();
      } else this.utilities.showAlert('invalidDate', 'pme::dataInicialMaior', 'error');
    }
  }
  
  public validaData() {
    const dataDe: ZdDate = Metadata.getInstance('DTDE');
    const dataAte: ZdDate = Metadata.getInstance('DTATE');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    if (dataAte.value !== null && !this.utilities.dateIsValid(dataDe.value, dataAte.value)) {
      this.utilities.showAlert('invalidDate', 'pme::dataInicialMaior', 'error');
      dataAte.value = '';
      grid.datasource.data = [];
    } else {
      this.reloadGridDestino();
    }
  }

  public returnDiaSemana(data: string) {
    const semana = ['dom', 'seg', 'ter', 'qua', 'qui', 'sex', 'sab'];
    let result = '';
    if (data) {
      const arr = data.split('/').reverse(); // ['year', 'month', 'day']
      const dateFormat = new Date(parseInt(arr[0], 10), parseInt(arr[1], 10) - 1, parseInt(arr[2], 10));
      const dia = dateFormat.getDay();
      result = I18n.translate(`pme::${semana[dia]}`);
    }
    return result;
  }
  
  public cancelEditingGridDest({ component }: IDictionary) {
    this.editable = false;
    const gridDestino = this.utilities.returnGrid(component) as GridEditable;
    if (gridDestino.editing) {
      this.gridEditController.newRow = false;
      this.gridEditController.cancelEditedRows({ component: gridDestino });
      gridDestino.datasource.currentRow = {};
      gridDestino.datasource.get();
    }
  }
  
  public addRow({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const row = grid.datasource.currentRow;

    Http.post('/returnDadosPadraoServ', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result;
      if (result.length > 0) this.gridEditController.addRow({ component: grid });
      else this.gridEditController.addRowSetFocusSecondPosition({ component: grid });
    });
  }
  
  public filtraTpCardDest() {
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    return [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralDest,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: 'T',
      },
    ];
  }
  
  public filtraServicoDestino() {
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const datasource = gridDestino.datasource.currentRow;
    
    return [
      {
        name: 'P_LISTATPCARD',
        operator: '=',
        value: datasource.TPCARD_DESTINO,
      },
      {
        name: 'P_TODOSTPCARD',
        operator: '=',
        value: 'T',
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: null,
      },
      {
        name: 'P_TGRUPTIPOCARD',
        operator: '=',
        value: 'T',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralDest,
      },
      {
        name: 'P_CDGRUPSERV',
        operator: '=',
        value: null,
      },
      {
        name: 'P_TCDGRUPSERV',
        operator: '=',
        value: 'T',
      },
    ];
  }

  public askFieldReplicationPer(fields: { [key: string]: any }) {
    DialogService.show({
      name: 'replicarValor',
      type: 'warning',
      title: 'pme::atencao',
      text: 'pme::desejaRepetirLinhas',
      buttons: [
        {
          name: 'acrescentar',
          component: 'ZdButton',
          label: 'pme::acrescentar',
          events: {
            click: () => {
              this.replicateFieldValueMultiplePeriodo('A', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'sobrepor',
          component: 'ZdButton',
          label: 'pme::sobrepor',
          events: {
            click: () => {
              this.replicateFieldValueMultiplePeriodo('S', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
  
  public replicateFieldValueMultiplePeriodo(option: string, fields: { [key: string]: any }) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const dataProcessamento = this.utilities.dtProcessFormatada;
  
    const parseDate = (str: string) => {
      const [day, month, year] = str.split('/').map(Number);
      return new Date(year, month - 1, day);
    };
  
    const formatDate = (date: Date) => {
      const day = String(date.getDate()).padStart(2, '0');
      const month = String(date.getMonth() + 1).padStart(2, '0');
      const year = date.getFullYear();
      return `${day}/${month}/${year}`;
    };
  
    const periodoDeDestinoInicio = parseDate(fields.DTDE_DESTINO);
    const editedRows = gridDestino.getEditedRows(false, true);
    const editedRowsMap = new Map(editedRows.map((row) => [row.PK, row]));
  
    gridDestino.datasource.data.forEach((row) => {
      if (row.PK !== gridDestino.datasource.currentRow.PK) {
        const periodoDeOrigemInicio = parseDate(row.DTINICARDPME);
        const periodoDeOrigemFim = parseDate(row.DTFINCARDPME);
  
        const diffInMs = periodoDeOrigemFim.getTime() - periodoDeOrigemInicio.getTime();
        const diffInDays = Math.ceil(diffInMs / (1000 * 60 * 60 * 24));
  
        const periodoAteDestino = new Date(periodoDeDestinoInicio);
        periodoAteDestino.setDate(periodoAteDestino.getDate() + diffInDays);
  
        if (periodoAteDestino < parseDate(dataProcessamento)) {
          this.utilities.showAlert('alert', 'pme::dataMenorIgualDtProcessa', 'warning');
          gridDestino.updateCell(row.PK, 'DTATE_DESTINO', null);
          gridDestino.updateCell(row.PK, 'DTATE_DESTINO', '');
        } else {
          const formattedDestino = formatDate(periodoAteDestino);
          const formattedInicio = formatDate(periodoDeDestinoInicio);
  
          if (option === 'S') {
            gridDestino.updateCell(row.PK, 'DTATE_DESTINO', formattedDestino);
            gridDestino.updateCell(row.PK, 'DTDE_DESTINO', formattedInicio);
          } else if (option === 'A') {
            const editedRow = editedRowsMap.get(row.PK);
            const dtAteDestinoValue = editedRow ? editedRow.DTATE_DESTINO : row.DTATE_DESTINO;
  
            if (dtAteDestinoValue === null || dtAteDestinoValue === undefined || dtAteDestinoValue === '') {
              gridDestino.updateCell(row.PK, 'DTATE_DESTINO', formattedDestino);
              gridDestino.updateCell(row.PK, 'DTDE_DESTINO', formattedInicio);
            }
          }
        }
      }
    });
  }
  
  private parseDate(str: string): Date {
    const [day, month, year] = str.split('/').map(Number);
    return new Date(year, month - 1, day);
  }

  private formatDate(date: Date): string {
    const day = String(date.getDate()).padStart(2, '0');
    const month = String(date.getMonth() + 1).padStart(2, '0');
    const year = date.getFullYear();
    return `${day}/${month}/${year}`;
  }
  
  public validaDiaSemanaDestEOrigem({ component }: IEventParam<ZdDate>) {
    const diaSemanaDestino = this.returnDiaSemana(component.value);
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const currentRow = gridDestino.datasource.currentRow;
    const diaSemanaOrigem = this.returnDiaSemana(currentRow.DTINICARDPME);
    const editedRows = gridDestino.getEditedRows(false, true);
    
    const parseDate = (str: string) => {
      if (!str) return null; // Verifica se a string é nula ou vazia
      const [day, month, year] = str.split('/').map(Number);
      return new Date(year, month - 1, day);
    };

    editedRows.forEach((row: any) => {
      if (row.PK === currentRow.PK) { // Verifica se é a linha sendo editada
        const dtDeDestino = parseDate(row.DTDE_DESTINO);
        const dtAteDestino = parseDate(row.DTATE_DESTINO);
  
        // Verifica se a data inicial de destino está dentro do período de destino
        const dtPerIniDestino = row.DTPERINI_DESTINO ? parseDate(row.DTPERINI_DESTINO) : null;
        
        if (dtPerIniDestino && dtDeDestino && dtAteDestino) {
          if (dtPerIniDestino >= dtDeDestino && dtPerIniDestino <= dtAteDestino) {
            if (diaSemanaDestino !== diaSemanaOrigem) {
              gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', null);
              gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', null);
              component.value = null;
              this.utilities.showAlert('alert', 'pme::diaSemanaDestinoOrigem', 'warning');
            } else {
              this.setDiaSemanaDest(gridDestino, diaSemanaDestino, component.value);
            }
          } else {
            gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', undefined);
            gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', undefined);
            this.utilities.showAlert('alert', 'pme::dataIncialNaoCompreendePeriodo', 'warning');
          }
        } else {
          gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', undefined);
          gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', undefined);
          this.utilities.showAlert('alert', 'pme::dataIncialNaoCompreendePeriodo', 'warning');
        }
      }
    });
  }
  
  public setDiaSemanaDest(grid: GridEditable, dia: string, dataInicial: string) {
    grid.updateCell(grid.datasource.currentRow.PK, 'DIASEMANA_DESTINO', dia !== '' ? dia : null);
    if (dia !== '') {
      this.askFieldReplication({
        DIASEMANA_DESTINO: dia,
        DTPERINI_DESTINO: dataInicial,
      });
    }
  }

  public askFieldReplication(fields: { [key: string]: any }) {
    DialogService.show({
      name: 'replicarValor',
      type: 'warning',
      title: 'pme::atencao',
      text: 'pme::desejaRepetirLinhas',
      buttons: [
        {
          name: 'acrescentar',
          component: 'ZdButton',
          label: 'pme::acrescentar',
          events: {
            click: () => {
              this.replicateFieldValueMultiple('A', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'sobrepor',
          component: 'ZdButton',
          label: 'pme::sobrepor',
          events: {
            click: () => {
              this.replicateFieldValueMultiple('S', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public replicateFieldValueMultiple(option: string, fields: { [key: string]: any }) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    
    const parseDate = (str: string) => {
      const [day, month, year] = str.split('/').map(Number);
      return new Date(year, month - 1, day);
    };
  
    if ('DSCARDPME_DESTINO' in fields) {
      const editedRowsMap = new Map(editedRows.map((row) => [row.PK, row]));
      gridDestino.datasource.data.forEach((row) => {
        const pk = row.PK;
        const editedRow = editedRowsMap.get(pk);
  
        if (option === 'S') {
          gridDestino.updateCell(pk, 'DSCARDPME_DESTINO', fields.DSCARDPME_DESTINO);
        } else if (option === 'A') {
          const descricao = editedRow ? editedRow.DSCARDPME_DESTINO : row.DSCARDPME_DESTINO;
          if (descricao === null || descricao === '') {
            gridDestino.updateCell(pk, 'DSCARDPME_DESTINO', fields.DSCARDPME_DESTINO);
          }
        }
      });
    } else {
      editedRows.forEach((row: any) => {
        const pk = row.PK;
        const dtDeDestino = parseDate(row.DTDE_DESTINO);
        const dtAteDestino = parseDate(row.DTATE_DESTINO);
        const diaSemanaOrigem = row.DIASEMANA;
  
        // Verifica se a data inicial de destino está dentro do período de destino
        const dtPerIniDestino = parseDate(fields.DTPERINI_DESTINO);
        if (dtPerIniDestino >= dtDeDestino && dtPerIniDestino <= dtAteDestino) {
          // Verifica se o dia da semana de destino é o mesmo que o da origem
          const diaSemanaDestino = fields.DIASEMANA_DESTINO;
          if (diaSemanaOrigem === diaSemanaDestino) {
            if (option === 'S') {
              gridDestino.updateCell(pk, 'DTPERINI_DESTINO', fields.DTPERINI_DESTINO);
              gridDestino.updateCell(pk, 'DIASEMANA_DESTINO', fields.DIASEMANA_DESTINO);
            } else if (option === 'A') {
              if (row.DTPERINI_DESTINO === null || row.DTPERINI_DESTINO === '') {
                gridDestino.updateCell(pk, 'DTPERINI_DESTINO', fields.DTPERINI_DESTINO);
                gridDestino.updateCell(pk, 'DIASEMANA_DESTINO', fields.DIASEMANA_DESTINO);
              }
            }
          }
        } else {
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', null);
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', '');
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', undefined);
        }
      });
    }
  }

  public periodoDestinoAte({ component }: IEventParam<ZdDate>) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const datasource = gridDestino.datasource.currentRow;
    const dataProcessamento = this.utilities.dtProcessFormatada;
  
    const parseDate = (str: string | null | undefined): Date | null => {
      if (!str) return null; // Verifica se a string é nula ou vazia
      const [day, month, year] = str.split('/').map(Number);
      return new Date(year, month - 1, day);
    };
  
    const formatDate = (date: Date) => {
      const day = String(date.getDate()).padStart(2, '0');
      const month = String(date.getMonth() + 1).padStart(2, '0');
      const year = date.getFullYear();
      return `${day}/${month}/${year}`;
    };
  
    const periodoDeOrigemInicio = parseDate(datasource.DTINICARDPME);
    const periodoDeOrigemFim = parseDate(datasource.DTFINCARDPME);
    const periodoDeDestinoInicio = parseDate(component.value);
  
    if (!periodoDeOrigemInicio || !periodoDeOrigemFim || !periodoDeDestinoInicio) {
      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTDE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTPERINI_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DIASEMANA_DESTINO', null);
      return;
    }
  
    const diffInMs = periodoDeOrigemFim.getTime() - periodoDeOrigemInicio.getTime();
    const diffInDays = Math.ceil(diffInMs / (1000 * 60 * 60 * 24));
  
    const periodoAteDestino = new Date(periodoDeDestinoInicio.getTime());
    periodoAteDestino.setDate(periodoAteDestino.getDate() + diffInDays);
  
    const dateProcessamento = parseDate(dataProcessamento);
    if (!dateProcessamento) {
      this.utilities.showAlert('alert', 'pme::invalidProcessDate', 'error');
      return;
    }
  
    // Verificar e limpar DTPERINI_DESTINO se estiver fora do intervalo
    const editedRows = gridDestino.getEditedRows(false, true);
    if (editedRows.length > 0) {
      editedRows.forEach((editedRow) => {
        if (editedRow.DTPERINI_DESTINO !== null && editedRow.DTPERINI_DESTINO !== undefined) {
          const dtPerIniDestino = parseDate(editedRow.DTPERINI_DESTINO);
          const dtDe = parseDate(editedRow.DTDE_DESTINO);
          const dtAte = parseDate(editedRow.DTATE_DESTINO);
  
          // Se a data inicial estiver fora do intervalo DTDE_DESTINO e DTATE_DESTINO, limpar os valores
          if (dtPerIniDestino && dtDe && dtAte && (dtPerIniDestino < dtDe || dtPerIniDestino > dtAte)) {
            gridDestino.updateCell(editedRow.PK, 'DTPERINI_DESTINO', undefined);
            gridDestino.updateCell(editedRow.PK, 'DTDE_DESTINO', undefined);
            gridDestino.updateCell(editedRow.PK, 'DTPERINI_DESTINO', '');
            gridDestino.updateCell(editedRow.PK, 'DIASEMANA_DESTINO', '');
          }
        }
      });
  
      const hasInvalidDtPerIni = editedRows.some((editedRow) => {
        if (editedRow.DTPERINI_DESTINO !== null && editedRow.DTPERINI_DESTINO !== undefined) {
          const dtPerIniDestino = parseDate(editedRow.DTPERINI_DESTINO);
          return dtPerIniDestino && (dtPerIniDestino < periodoDeDestinoInicio || dtPerIniDestino > periodoAteDestino);
        }
        return false; // Retorno explícito de false para DTPERINI_DESTINO nulo ou undefined
      });
  
      if (hasInvalidDtPerIni) {
        return;
      }
    }
  
    if (periodoAteDestino < dateProcessamento) {
      this.utilities.showAlert('alert', 'pme::dataMenorIgualDtProcessa', 'warning');
      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTDE_DESTINO', null);
      component.value = null;
    } else {
      const formattedInicio = formatDate(periodoDeDestinoInicio);
      const formattedFim = formatDate(periodoAteDestino);
  
      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', formattedFim);
  
      this.askFieldReplicationPer({
        DTDE_DESTINO: formattedInicio,
        DTATE_DESTINO: formattedFim,
      });
    }
  }

  public async saveEditingGridDest() {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    
    let allRowsValid = true;
    
    editedRows.forEach((editedRow: any) => {
      if (!this.isRowFilled(editedRow)) {
        allRowsValid = false;
        // Para a iteração se encontrar uma linha inválida
      }
    });
  
    if (allRowsValid) {
      gridDestino.editing = false;
      const toolbarSlot = gridDestino.toolbarSlot;
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = !e.disabled;
        if (e.name === 'saveButton') e.disabled = !e.disabled;
      });
      this.gridEditController.isInlineEditing = gridDestino.editing;
      
      await gridDestino.saveEditedRows();
      const data = [...gridDestino.datasource.data];
      await gridDestino.datasource.updateData([]);
      await gridDestino.datasource.updateData(data);
    } else {
      this.utilities.showAlert('alertError', 'pme::verificarCamposObg', 'error');
    }
  }
  
  private isRowFilled(row: any): boolean {
    return !!(
      row.TPCARD_DESTINO
      && row.CDSERVFILI_DESTINO
      && row.DTDE_DESTINO
      && row.DTATE_DESTINO
      && row.DTPERINI_DESTINO
      && row.DIASEMANA_DESTINO
      && row.DSCARDPME_DESTINO
    );
  }
  
  public changeCardDest({ component }: IEventParam<Select>) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    if (component) {
      gridDestino.updateCell(gridDestino.datasource.currentRow.PK, 'CDSERVFILI_DESTINO', '');
    }
  }
  
  public filtraServicoDestinoFocus({ component }: IEventParam<Select>) {
    component.datasource.data = [];
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    if (editedRows.length > 0) {
      editedRows.forEach((row: any) => {
        const filter = [
          {
            name: 'P_LISTATPCARD',
            operator: '=',
            value: row.TPCARD_DESTINO,
          },
          {
            name: 'P_TODOSTPCARD',
            operator: '=',
            value: row.TPCARD_DESTINO !== '' ? null : 'T',
          },
          {
            name: 'P_CDGRUPTIPOCARD',
            operator: '=',
            value: null,
          },
          {
            name: 'P_TGRUPTIPOCARD',
            operator: '=',
            value: 'T',
          },
          {
            name: 'P_CDFILCENTPME',
            operator: '=',
            value: centralDest,
          },
          {
            name: 'P_CDGRUPSERV',
            operator: '=',
            value: null,
          },
          {
            name: 'P_TCDGRUPSERV',
            operator: '=',
            value: 'T',
          },
        ];
        component.datasource.filter = filter;
        component.datasource.get();
      });
    } else {
      const datasource = gridDestino.datasource.currentRow;
      
      const filter = [
        {
          name: 'P_LISTATPCARD',
          operator: '=',
          value: datasource.TPCARD_DESTINO,
        },
        {
          name: 'P_TODOSTPCARD',
          operator: '=',
          value: datasource.TPCARD_DESTINO !== null ? null : 'T',
        },
        {
          name: 'P_CDGRUPTIPOCARD',
          operator: '=',
          value: null,
        },
        {
          name: 'P_TGRUPTIPOCARD',
          operator: '=',
          value: 'T',
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: centralDest,
        },
        {
          name: 'P_CDGRUPSERV',
          operator: '=',
          value: null,
        },
        {
          name: 'P_TCDGRUPSERV',
          operator: '=',
          value: 'T',
        },
      ];
      component.datasource.filter = filter;
      component.datasource.get();
    }
  }
  
  public clearServDest() {
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = grid.getEditedRows(false, true);
    
    const currentRow = grid.datasource.currentRow;
    const editedRow = editedRows.find((row) => row.PK === currentRow.PK);
    
    if (editedRow) {
      grid.updateCell(editedRow.PK, 'CDSERVFILI_DESTINO', null);
      grid.updateCell(editedRow.PK, 'CDSERVFILI_DESTINO', undefined);
    }
  }

  public replaceDesc({ component }: IEventParam<TextInput>) {
    if (component.value.length > 0 && component.value.length <= 40) {
      this.askFieldReplication({
        DSCARDPME_DESTINO: component.value,
      });
    }
  }
  
  public setGridDestino() {
    const formCentralDest = Metadata.getInstance<Form>('formCentralDestino');
    const formDadosOrigem = Metadata.getInstance<Form>('formDadosOrigem');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    
    const dataToSend = {
      formCentralDest: formCentralDest.value,
      formDadosOrigem: formDadosOrigem.value,
    };

    Http.post('/setDataSourceGridDestino', dataToSend).then((response: IDictionary) => {
      const result = response.data.dataset.result;
      if (result.length > 0) {
        grid.datasource.updateData(result);
      } else {
        grid.datasource.updateData([]);
      }
    });
  }
  
  public exportar() {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const rows = gridDestino.selectedRows;
    const editedRows = gridDestino.getEditedRows(false, true);
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;

    const areAllFieldsFilled = (row: any) => {
      const requiredFields = [
        'TPCARD_DESTINO', 'CDSERVFILI_DESTINO', 'DTDE_DESTINO', 'DTATE_DESTINO',
        'DTPERINI_DESTINO', 'DIASEMANA_DESTINO', 'DSCARDPME_DESTINO',
      ];
  
      return requiredFields.every((field) => row[field] !== null && row[field] !== undefined);
    };
  
    const combinedRows = rows.map((selectedRow) => {
      const foundEditedRow = editedRows.find((er) => er.PK === selectedRow.PK);
      const combinedRow = foundEditedRow ? { ...selectedRow, ...foundEditedRow } : selectedRow;
      return { ...combinedRow, CENTRAL_DESTINO: centralDest, CENTRAL_ORIGEM: centralOrigem };
    });
  
    if (combinedRows.length > 0) {
      if (combinedRows.every(areAllFieldsFilled)) {
        LoadingService.show(' ');
        Http.post('/verificaCardapioPeriodo', { requestType: 'Row', rows: combinedRows }).then((response) => {
          const result = response.data.dataset.result;
          let desc = '';
          const ate = I18n.translate('pme::ate');
          if (result.length > 0) {
            result.forEach((row: any) => {
              desc += `${row.DTINICARDPME} ${ate} ${row.DTFINCARDPME} `;
            });
          
            this.utilities.showAlert(
              'alertError',
              I18n.translate('pme::periodoCompreendeJaCadastrado', { PARAM1: desc }),
              'error',
            );
          } else {
            Http.post('/verificaConfigCardServ', { requestType: 'Row', rows: combinedRows }).then((configPadrao) => {
              const resultConf = configPadrao.data.dataset.result;
              if (resultConf.length > 0) {
                combinedRows.forEach((row) => {
                  if (this.utilities.dateIsValid(row.DTDE_DESTINO, row.DTATE_DESTINO)) {
                    Http.post('/insertListCardAssisIntegra', { requestType: 'Row', rows: combinedRows }).then(() => {
                      this.utilities.showAlert('alertError', 'pme::exportadoComSucesso', 'green');
                    }).finally(() => {
                      LoadingService.hide();
                    });
                  } else {
                    this.utilities.showAlert('invalidDate', 'pme::dataInicialMaior', 'error');
                  }
                });
              } else {
                this.utilities.showAlert('alertError', 'pme::servicoSemConfig', 'red');
              }
            });
          }
        }).finally(() => {
          LoadingService.hide();
        });
      } else {
        this.utilities.showAlert('alertError', 'pme::verificarCamposObg', 'error');
      }
    } else {
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    }
  }
}
